/*
  ALPHA interface for CHESS

  Revision: 15-01-96

  Copyright (C) 1986, 1987, 1988, 1989, 1990 Free Software Foundation, Inc.
  Copyright (c) 1988, 1989, 1990  John Stanback
  Copyright (c)  1990, 1991   David Giunti
  Copyright (c) 1996  Yves Debilloez

  This file is part of CHESS.

  CHESS is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY.  No author or distributor accepts responsibility to anyone for
  the consequences of using it or for whether it serves any particular
  purpose or works at all, unless he says so in writing.  Refer to the CHESS
  General Public License for full details.

  Everyone is granted permission to copy, modify and redistribute CHESS, but
  only under the conditions described in the CHESS General Public License. A
  copy of this license is supposed to have been given to you along with
  CHESS so you can know your rights and responsibilities.  It should be in a
  file named COPYING.  Among other things, the copyright notice and this
  notice must be preserved on all copies.

   v30b improvements   dag
   added INT10 preproc. to allow faster screen update
   added Gnuchess, and version listing during play
   fixed bug in both module to allow 'force' setup then play Both from
     that position without reentry of force command!
   v30c improvs   dag
    cheekie box anounces Check!
   eliminated case sensativity on user inputs
   added Lurk option turns on post, off bell, plays both sided
   made some progress in recovery from answering no to the Abort message
     needs better critical error handler..
   v30d
     compiler switches improvements,
   /PACKC /FARC /E /STACK:0x2020 link options
    ^  packs the code, just checks for optimal near calls
            ^ more code packing for near pointers
              the above two were for compact compiles, but they
              shouldn't hurt small model
                 ^ packs .EXE to smallest possible size
                      ^ 8K stack
   did some work in the signal() calls to get compilation with
     default pascal compilation, but signal would not work with
     this option.. since It didn't break anything, I didn't remove
     some of these changes.
   added conditional code to implement the pascal calls for all
     GC functions, MSC 6 users may want to experiment by
     defining Px _fastcall for faster .EXEs
   moved to small model, far data conversions all conditional
   add int10 clear to eoln and clearscreen ANSI.sys no longer needed
   added current active book and hashfile status to display
     Hash is the Ram only  Hash+ is the disk file active
   moved positions of output in edit mode to go with new display
     stats.
   repositioned cursor to show status of save message to not
      dirupt display
   added cursor positioning in exitchess to show board after exit
       30e
   changed position and format of the abort message, conditional
     to help eliminate display confusion if ^C at enter prompt
   corrected current book showing
   allows comments after moves in .boo format
     a comment must begin after a ; character and continues to the
     end of that line
     ! is now the sole method to start a new book entry.
   added min and max parameters to show depth, now the a/b window is
     displayed at start of the level's search
   lowercased awindow and bwindow to conform to all lowercase
     Your Move entries.  To allow experimentation.
   undefined REGISTER to allow optomizer to do the allocations
      Can #define REGISTER register  to get it back
   added selection for n moves in x minutes to the level selection
     so command line form is no longer needed.
     Added Depth selection to this menu and a setting for vicious.

   corrected Level display in Help menu.  Now displays correct seconds
     if TCflag is 0.. ie 1 move in any time period
   TerminateSearch *IS* working!  I just didn't understand that it
     should not just exit, but terminate the current search.
   printz remains printf , tried cprintf and fprintf(stdout
     which could be slightly faster on some systems, but testing revealed
     that printf played a better game! this could be code size related???
   added global variable numbooks to display number of books in current
     read in book
   changed -a Your move prompt to reflect thinking on opponents time
        removed check announcement which was not working correctly yet
          added conditional CHECKANO to possibly reactiveate
   corrected the a typo from the original port that only allowed 50
      moves per opening book.. the allocated buffer should have been
      0x100 and not just 100 .. needs to be able to handle 128 full
      moves to keep to original spec. and not trash the stack
   added promotion message to the help list
   added title 'T' routine, and the i<3 eater is not really a bad
      move eater.. sort of a catch all eater.  The 30 book reader
      reported 289 games but this was an error!  There are 285
      not quite sure why this was but it is fixed now...
      to parse openings
   added display of book name while reading it in
   Trying to add a clutch to the quite mode search via resetting
     Level when more than 1 move per time period is selected in
     the new 11 level.  This had no effect so I will comment out
     the code section in selectlevel.
   added display of a character passed when ShowResults is called
     overlooked in the original dos port.  fixed routine to show all
     possible plys.  Move test for (post active) back to gnuchess.c
     will save time when post is not selected.
        looks like a bug somewhere is causeing moves 26 to 29 to
        not be printed.
   added position command at your_move prompt.  Reads in an ascii
     file of edit commands.  usefull for different board setups.
     comments are allowed in this file. no extension needed.
     added entry in help menu
 30f
   corrected format for Max Tree, now shows correct number.
   added access to ToFileAt at the Your Move prompt.  Allows
     user to select depth at which hashfile references happen
     (is at this number +1 )
   changed the HASH command from Your Move to turn the Hashfile
     on and off only.  Can not shut off the table.
   added neg-ahead logic to seteasy()
   work on Position command.
     Added 't' parameter to reset clock. Since the clock is started
       when any command is given the clock starts, this allows more
       accuate benching
     Added 's' parameter.  copies next set of characters to the your
        move string to execute that command.  good for both or lurk
        commands.  made global char s[80] from input command and renamed
        to str[] to implement this command easily. Could have been
        problems with the way my parser was handeling the string
        after s, but redid as a switch, and limited commands to
        b, black, w white, s (self) both, and l lurk.
   added overwrite protection to the savegame subsection to prevent
     overwriting existing file.  Just aborts of overwrite not wanted
     should be made a global function for general use and allow
     rentry of name for other write functions.. along with with
     removal of other get filename and write subsections.

   would be nice  and todo list
   redo .time to unsigned so the printing of chess.lst is correct
     for deep searches.
   rework depth display to show quiesent search depth
   linked list with ratings for the book moves, to tree the book
   add free or something for malloc()ed book.. this is sloppy


*/
char gname[] = "Dave\'s GnuChess";
char ver[] = "30f(aa)" ;        /* version of this system */

#include <stdio.h>
#ifndef MSDOS  /* duped definitions */
extern int fclose ();
extern int fscanf ();
extern int fprintf ();
extern int fflush ();
#endif

extern short InChk;     /*in gnuchess.c  to report check  */
extern FILE *hashfile;  /* "  to display hashfile usage */
extern short rehash,    /* to mess with hashfile storage */
             vicious,   /* bool in gc  controls such behaviour */
             ToFileAt,  /* holds level of hashfile access */
				 ahead;     /* command line status of computer thinking */
#include <ctype.h>

#ifdef MSDOS
#include <dos.h>
#include <stdlib.h>
#include <time.h>
#include <conio.h>     /* for cprintf() */

#define ESC 0x1B
#define scanz scanf
#define printz printf
#define INT10  1       /* turn on inline gotoxys */

#define refresh() fflush(stdout)


/* try to integrate the signal system... only sigint is listed
   so try only that first.. leaving the sigquit for later.
   sigquit is a unix function that is related to multi-processing
   systems.  MSC 5.1 doesn't know about it so it is not needed.  dag
*/
#include <malloc.h>
#include <stdlib.h>

#include <signal.h>
#include <process.h>
void  TerminateSearch (),  Die ();  /* processes pointed at by signal */

  /* lets try direct cursor bios movement for gotoxy */
#include <dos.h>     /* for regs & int86 defs */
union REGS r1 = {0x0200, 0x0000,0,0};  /* in  regs for gotoXY
                                             ah = 2 , 00 current page */
union REGS  r2 ;  /*  out regs for reg returns */
   /* regs for clear to end of line */
union REGS  getpos = {0x0300,0,0,0};     /* in regs for get cursor position */
union REGS  clrln = {0x0920,0,0,0};    /* in regs to clear line */

#define Px  pascal   /* use optimal calling convention */
#define Fr  far      /* use far calls for large data */
#define malloc _fmalloc   /* needs far heap to allocate book in small
                             memory model for dos */
#define Hu  huge      /* define huge for book moves */
        /* if huge is needed in the display code remember that it can
           be redefined, as needed in GC.    dag */
#define REGISTER      /* undefine to allow optomizer to do it */

/* a couple of function prototypes to try and coerse far addresses from
   near data if function calls dont work */
/*   done on function call, but I am told that msc 4 may need them so
     here they hide ... just in case   dag
void Px MakeMove (short int, struct leaf Fr *, short int *,\
                     short int *, short int *, short int *) ;
void Px UnmakeMove (short int, struct leaf Fr *, short int *,\
                     short int *, short int *, short int *) ;
*/
#else    /* It is not MSDOS */
#define Px           /* null it if not in MSC */
#define Fr
#define Hu
#define REGISTER register     /* turn it back on for non reg coloring
                                c compilers */
#include <sys/param.h>
#include <sys/types.h>
#include <sys/file.h>
#include <curses.h>
#include <signal.h>

#define scanz fflush(stdout),scanw
#define printz printw

void TerminateSearch (), Die ();
#endif /* MSDOS */

#ifndef MSDOS   /* duped definitions */
/* <stdlib.h> */
extern void *malloc ();
extern void exit ();
/* <time.h> */
extern long int time ();
/* <string.h> */
extern char *strcat ();
extern int strcmp ();
extern char *strcpy ();
#endif

#include "gnuchess.h"

#ifdef __BORLANDC__
#include "declare.h"
int strcmp(const char *s1, const char *s2);
char *strcpy(char *dest, const char *src);
#endif

#ifndef BOOK
#define BOOK "gnuchess.boo"  /* was "/usr/games/lib/gnuchess.boo" */
#endif
char bk_title[128] = BOOK;  /* allow for changing titles and path */
short numbooks;             /* for number of titles in the book */
char cur_book[81];           /* to hold current book title.. smaller
                               later   dag */
#define pxx " PNBRQK"
#define qxx " pnbrqk"
#define rxx "12345678"
#define cxx "abcdefgh"

static char mvstr[5][6];
static long evrate;

char str[80]; /* removed string entry array s and renamed str from
                 input command to allow strings to be passed back
                 from edit board easily     dag */



void Px
Initialize ()
{
  signal (SIGINT, Die);
#ifndef MSDOS
  signal (SIGQUIT, Die);
  initscr ();
  crmode ();
#else   /* is msdos */
 /* initialize for screen parameters here */
#ifdef INT10
  r2.x.ax = 0x0f00;                /* set call to get crt modes */
  int86(0x10,&r2,&r2);
  getpos.h.bh = clrln.h.bh = r2.h.bh ; /* set correct active page */
  r2.x.ax = 0x0800 ;  /* set to get current character/attributes    */
  int86(0x10,&r2,&r2);
  clrln.h.bl = r2.h.ah;           /* set common attribute */

#endif    /* int10 active */

#endif /* MSDOS */
}

void Px
ExitChess ()
{
  if (hashfile)     /*added   dag */
      fclose(hashfile);
  ListGame();
#ifndef MSDOS
  nocrmode ();
  endwin ();
#else    /* is msdos */
  gotoXY(0,24);   /* position cursor to leave the board on the screen */
#endif /* MSDOS */

  exit (0);
}

void Px
surelow(target)
       char *target;
{
  while (*target)
  {  if (isalpha(*target) )
         *target = tolower(*target);
	  ++target ; /* point at next */
  }
}

void Px
ShowMessage (s)
     char *s;
{
  gotoXY(48,23);   /* changed from 48,23 to avoid screen hop */
  printz( "%s", s);
  ClrEoln ();
}

void Px
ClearMessage ()
{
  gotoXY(48,23);    /* was 48, 24   changed to actually clear */
  ClrEoln ();
}

#ifdef MSDOS
void Die ()
#else
void  Die ()
     int sig;
#endif
{
  char s[80];
  signal (SIGINT, (SIG_IGN));
#ifndef MSDOS
  signal (SIGQUIT, SIG_IGN);
#endif

  ShowMessage ("Abort? ");
  scanz ("%s", s);
  surelow(s);
  if (strcmp (s, "yes") == 0)
    ExitChess ();

#ifdef MSDOS
  ClearMessage ();
  gotoXY(48,19);
  printz( "Press enter then BD to continue"); /* required to regain control
               with the way signal works in MsDos, can`t driectly return! */
#endif

  signal (SIGINT, Die);
#ifndef MSDOS
  signal (SIGQUIT, Die);
#endif
return ;

}

#ifdef MSDOS
void
TerminateSearch ()
#else
void
TerminateSearch (sig)
     int sig;
#endif
{
  signal (SIGINT, ( SIG_IGN));
#ifndef MSDOS
  signal (SIGQUIT, SIG_IGN);
#endif
  timeout = true;
  bothsides = false;
  signal (SIGINT, Die);
#ifndef MSDOS
  signal (SIGQUIT, Die);
#endif
}

void Px
algbr (f, t, flag)
     short int f;
     short int t;
     short int flag;


/*
   Generate move strings in different formats, a hook has been provided for
   underpromotion
*/

{
  int m3p;

  if (f != t)
    {
      /* algebraic notation */
      mvstr[0][0] = cxx[column (f)];
      mvstr[0][1] = rxx[row (f)];
      mvstr[0][2] = cxx[column (t)];
      mvstr[0][3] = rxx[row (t)];
      mvstr[0][4] = '\0';
      mvstr[3][0] = '\0';
      if ((mvstr[1][0] = qxx[board[f]]) == 'p')
	{
	  if (mvstr[0][0] == mvstr[0][2])	/* pawn did not eat */
	    {
	      mvstr[2][0] = mvstr[1][0] = mvstr[0][2];	/* to column */
	      mvstr[2][1] = mvstr[1][1] = mvstr[0][3];	/* to row */
	      m3p = 2;
	    }
	  else
	    /* pawn ate */
	    {
			mvstr[2][0] = mvstr[1][0] = mvstr[0][0];	/* from column */
	      mvstr[2][1] = mvstr[1][1] = mvstr[0][2];	/* to column */
	      mvstr[2][2] = mvstr[0][3];
	      m3p = 3;		/* to row */
	    }
	  mvstr[2][m3p] = mvstr[1][2] = '\0';
	  if (flag & promote)
	    {
          /*****  this routine adds a  q b r n to the movestrings
              dag ****/

          mvstr[0][4] = mvstr[1][2] = mvstr[2][m3p] = qxx[flag & pmask];
	      mvstr[1][3] = mvstr[2][m3p + 1] = mvstr[0][5] = '\0';

	    }
	}
      else
	/* not a pawn */
	{
	  mvstr[2][0] = mvstr[1][0];
	  mvstr[2][2] = mvstr[1][1] = mvstr[0][2];	/* to column */
	  mvstr[2][3] = mvstr[1][2] = mvstr[0][3];	/* to row */
	  mvstr[2][4] = mvstr[1][3] = '\0';
	  mvstr[2][1] = mvstr[0][1];
	  strcpy (mvstr[3], mvstr[2]);
	  mvstr[3][1] = mvstr[0][0];
	  if (flag & cstlmask)
	    {
	      if (t > f)
		{
		  strcpy (mvstr[1], "o-o");
		  strcpy (mvstr[2], "O-O");
		}
	      else
		{
		  strcpy (mvstr[1], "o-o-o");
		  strcpy (mvstr[2], "O-O-O");
		}
	    }
	}
    }
  else
    mvstr[0][0] = mvstr[1][0] = mvstr[2][0] = mvstr[3][0] = '\0';
}

void Px
seteasy (call_param)   /* toggles thinking on users time, false is on dag */
short call_param;      /* to trigger default setting off from your move */
{
  easy = !easy;
  if (call_param)      /* 1 from your move, 0 from GC    */
      ahead = !ahead;  /* turn off command line trigger too   dag */
}

int Px
VerifyMove (s, iop, mv)
     char *s;
     short int iop;
     short unsigned int *mv;




/*
   Compare the string 's' to the list of legal moves available for the
   opponent. If a match is found, make the move on the board.
*/

{
  static short pnt, tempb, tempc, tempsf, tempst, cnt;
  static struct leaf xnode;
  struct leaf Fr *node;

  *mv = 0;
  if (iop == 2)
    {
#ifdef MSDOS
      UnmakeMove (opponent,(struct leaf Fr *) &xnode, &tempb, &tempc, &tempsf, &tempst);
#else
      UnmakeMove (opponent, &xnode, &tempb, &tempc, &tempsf, &tempst);
#endif
      return (false);
    }
  cnt = 0;
  MoveList (opponent, 2);
  pnt = TrPnt[2];
  while (pnt < TrPnt[3])
    {
		node = &Tree[pnt++];
      algbr (node->f, node->t, (short) node->flags);
      if (strcmp (s, mvstr[0]) == 0 || strcmp (s, mvstr[1]) == 0 ||
	  strcmp (s, mvstr[2]) == 0 || strcmp (s, mvstr[3]) == 0)
    {
	  cnt++;
	  xnode = *node;
	}
    }
  if (cnt == 1)
    {
#ifdef MSDOS
      MakeMove (opponent,(struct leaf Fr *) &xnode, &tempb, &tempc, &tempsf, &tempst);
#else
      MakeMove (opponent, &xnode, &tempb, &tempc, &tempsf, &tempst);
#endif
      if (SqAtakd (PieceList[opponent][0], computer))
	{
#ifdef MSDOS
      UnmakeMove (opponent,(struct leaf Fr *) &xnode, &tempb, &tempc, &tempsf, &tempst);
#else
      UnmakeMove (opponent, &xnode, &tempb, &tempc, &tempsf, &tempst);
#endif
	  ShowMessage ("Illegal Move!!");
	  return (false);
	}
      else
	{
	  if (iop == 1)
	    return (true);
	  if (xnode.flags & epmask)
	    UpdateDisplay (0, 0, 1, 0);
	  else
	    UpdateDisplay (xnode.f, xnode.t, 0, (short) xnode.flags & cstlmask);
	  if (xnode.flags & cstlmask)
	    Game50 = GameCnt;
	  else if (board[xnode.t] == pawn || (xnode.flags & capture))
	    Game50 = GameCnt;
	  GameList[GameCnt].depth = GameList[GameCnt].score = 0;
	  GameList[GameCnt].nodes = 0;
	  ElapsedTime (1);
	  GameList[GameCnt].time = (short) et;
	  TimeControl.clock[opponent] -= et;
	  --TimeControl.moves[opponent];
	  *mv = (xnode.f << 8) + xnode.t;
	  algbr (xnode.f, xnode.t, false);
	  return (true);
	}
    }
  if (cnt > 1)
    ShowMessage ("Ambiguous Move!");
  return (false);
}


void Px
EditBoard (xc)
  short xc;     /* added parameter to trigger get position routines    dag */

/*
   Set up a board position. Pieces are entered by typing the piece followed
   by the location. For example, Nf3 will place a knight on square f3.
*/

{
  short a, r, c, sq, ch;
  short ctr;
  char s[80];
  FILE *pfd;
  char pfname[40];    /* to hold a path and file name */

  if (xc)               /* added for get position  dag */
   {
    ShowMessage ("Enter file name: ");
    scanz ("%s", pfname);
    if ((pfname[0] == '\0') || ((pfd = fopen (pfname, "r")) == NULL) )
         xc = 0;   /* turn off file useage */
    }
  ClrScreen ();
  UpdateDisplay (0, 0, 1, 0);
  gotoXY(48,4);
  printz( ".   Exit to main");
  gotoXY(48,5);
  printz( "#   Clear board");
  gotoXY(48,6);
  printz( "c   Change sides");
  gotoXY(48,9);
  printz( "Enter piece & location: ");
  a = white;
  do
    {
      gotoXY(48,7);
      printz( a == white ? "Editing:  white" : "Editing:  black");
      gotoXY(72,9);
      ClrEoln ();
      if (xc)                            /* added to parse position file  dag */
        {  while ( (ch = getc (pfd)) == ' ');     /* eat opening white spaces */
            ctr = 0;                              /* init counter */
            s[0] = ch;                             /* transfer first char */
            while (ch != ';' && ch != ' ' && ch != '\n' && ch != EOF) /*limits*/
                s[++ctr] = ch = getc (pfd);                /* read in temp */
            if ( ch == ';')
                 while (ch != '\n' && ch != EOF)
                     ch = getc (pfd);              /* eat comment */
            s[++ctr] = '\0';            /* end string, not really needed */
            if (ch == EOF || s[0] == '.')
             {  fclose(pfd);
                xc = 0;    /* turn off read in function if file */
             }
           printz("%s",s);
			}
      else   /* it is a normal edit */
      {
        scanz ("%s", s);
        surelow(s);
      }
      if (s[0] == '#')
       {
         for (sq = 0; sq < 64; sq++)
         {
	      board[sq] = no_piece;
	      color[sq] = neutral;
         }
         UpdateDisplay (0, 0, 1, 0);
       }
        /* add time clearing code here    dage */
      if (s[0] == 't' || s[0] == 'T')
       {  /* reset clock to 0 for testing  */
          time0 = time ((long *) 0);
          et0 =0;
       }
        /* add routine to copy string to you move string */
		if (s[0] == 's' || s[0] == 'S')
       {
          switch (s[1])
          {   case 'b':
              case 'B':  strcpy( str, "black");
                         break;
              case 'w':
              case 'W':  strcpy( str, "white");
                         break;
              case 's':
              case 'S':  strcpy( str, "both");
                         break;
              case 'l':
              case 'L':  strcpy(str, "lurk");
              default:   break;
           }
       }

      if (s[0] == 'c' || s[0] == 'C')
          a = otherside[a];
      c = s[1] - 'a';
      r = s[2] - '1';
		if ((c >= 0) && (c < 8) && (r >= 0) && (r < 8))
       {
        sq = locn (r, c);
        color[sq] = a;
        if (s[0] == 'p')
          board[sq] = pawn;
        else if (s[0] == 'n' || s[0] == 'N')
                 board[sq] = knight;
             else if (s[0] == 'b' || s[0] == 'B')
                      board[sq] = bishop;
                  else if (s[0] == 'r' || s[0] == 'R')
                           board[sq] = rook;
                       else if (s[0] == 'q' || s[0] == 'Q')
                                 board[sq] = queen;
                            else if (s[0] == 'k' || s[0] == 'K')
                                      board[sq] = king;
                                 else
                                  {
                                   board[sq] = no_piece;
                                   color[sq] = neutral;
                                  }
        DrawPiece (sq);
		 } /* end if it is a piece */
  } while (s[0] != '.');
  for (sq = 0; sq < 64; sq++)
    Mvboard[sq] = (board[sq] != Stboard[sq]) ? 10 : 0;
  GameCnt = -1;
  Game50 = 0;
  Sdepth = 0;
  InitializeStats ();
  ClrScreen ();
  UpdateDisplay (0, 0, 1, 0);
}

void Px
help ()
{
  ClrScreen ();
  gotoXY(28,1);
  printz( "CHESS command summary");
  gotoXY(1,3);
  printz( "g1f3      move from g1 to f3");
  gotoXY(1,4);
  printz( "nf3       move knight to f3");
  gotoXY(1,5);
  printz( "o-o       castle king side");
  gotoXY(1,6);
  printz( "o-o-o     castle queen side");
  gotoXY(1,7);
  printz( "c7c8q or b,n,r to promote pawn");
  gotoXY(1,8);
  printz( "switch    sides with computer");
  gotoXY(1,9);
  printz( "white     computer plays white");
  gotoXY(1,10);
  printz( "black     computer plays black");
  gotoXY(1,11);
  printz( "reverse   board display");
  gotoXY(1,12);
  printz( "both      computer match");
  gotoXY(1,13);
  printz( "random    randomize play");
  gotoXY(1,14);
  printz( "undo      undo last move");
  gotoXY(1,15);
  printz( "lurk    post, beep, both macro");
  gotoXY(42,3);
  printz( "level     change level");
  gotoXY(42,4);
  printz( "depth     set search depth");
  gotoXY(42,5);
  printz( "post      principle variation");
  gotoXY(42,6);
  printz( "hint      suggest a move");
  gotoXY(42,7);
  printz( "bd        redraw board");
  gotoXY(42,8);
  printz( "force     enter game moves");
  gotoXY(42,9);
  printz( "list      game to chess.lst");
  gotoXY(42,10);
  printz( "save      game to file");
  gotoXY(42,11);
  printz( "get       game from file");
  gotoXY(42,12);
  printz( "new       start new game");
  gotoXY(42,13);
  printz( "quit      exit CHESS");
  gotoXY(42,13);
  printz( "edit      edit board");
   gotoXY(42,14);
  printz( "position  get ascii edit file");



  gotoXY(10,21);
  printz( "Computer: ");
  if (computer == white)
    printz( "WHITE");
  else
    printz( "BLACK");
  gotoXY(10,22);
  printz( "Opponent: ");
  if (opponent == white)
    printz( "WHITE");
  else
    printz( "BLACK");
  gotoXY(10,23);
  printz( "Level: %ld sec.", TCflag ? (Level * (long)TCminutes): Level);
  gotoXY(10,24);
  printz( "Easy mode: ");
  if (easy)
    printz( "ON");
  else
    printz( "OFF");
  gotoXY(40,21);
  printz( "Depth: %d", MaxSearchDepth);
  gotoXY(40,22);
  printz( "Random: ");
  if (dither)
    printz( "ON");
  else
    printz( "OFF");
  gotoXY(40,23);
  printz( "Transposition table: ");
  if (hashflag)
    printz( "ON");
  else
    printz( "OFF");
  gotoXY(40,24);
#ifdef MSDOS
  printz( "Hit <1> to return: ");
#else
  printz( "Hit <ESC> to return: ");
#endif
  refresh ();
#ifdef MSDOS
  while (getchar () != '1') ;
#else
  while (getchar () != 27) ;
#endif
  ClrScreen ();
  UpdateDisplay (0, 0, 1, 0);
}

void Px
ShowDepth (ch, min, max)   /* added min & max to show alpha beta window dag */
     char ch;
     short min, max;
{
  gotoXY(48,4);
  printz( "Depth= %d%c  %d<>%d", Sdepth, ch, min, max);
  ClrEoln ();
}

void Px
ShowResults (score, bstline, ch)
     short int score;
     short unsigned int *bstline;
     char ch;
{
  short d, e, ply;
      /* removed if (post) to calling routines, 2 in gnuchess
         for faster play in non post modes dag */
      e = lpost;  /* lpost global, in GC, assigned at end of this
                     procedure, last post position line  dag*/
      gotoXY(48,5);                       /* score showing */
      printz( "Score= %d%c", score, ch);
      ClrEoln ();
      d = 8;                /* first line printing posted bestline
                               if d needs changing update newgame in gc*/
      gotoXY(48,d);
      ClrEoln ();
      for (ply = 1; bstline[ply] > 0; ply++)
	{
	  algbr ((short) bstline[ply] >> 8, (short) bstline[ply] & 0xFF, false);
	/*   if (ply == 5 || ply == 9 || ply == 13 || ply == 17)  old   dag */
      if (ply == 6 || ply == 11 || ply == 16 || ply == 21 || ply == 26)
        {
          gotoXY(48,++d);        /* if full goto next and clear it first */
	      ClrEoln ();
	    }
      printz( "%5s ", mvstr[0]);       /* print each ply one by one */
    }  /* end for bstline */
    ClrEoln ();       /* clear non printed part of line */
    lpost = d;        /* update lastposted + 1 */
    while (++d <= e)
     {
       gotoXY(48,d);
       ClrEoln ();
     }
}

void Px
SearchStartStuff (side)
     short int side;
{
  short i;
  signal (SIGINT, (TerminateSearch));
#ifndef MSDOS
  signal (SIGQUIT, TerminateSearch);
#endif /* MSDOS */
  if (player == computer)
    for (i = 5; i < 14; i++)
      {
    gotoXY(48,i);
	ClrEoln ();
      }
}

void Px
OutputMove ()
{
  int i;

  if (root->flags & epmask)
    UpdateDisplay (0, 0, 1, 0);
  else
    UpdateDisplay (root->f, root->t, 0, (short) root->flags & cstlmask);
  gotoXY(48,17);
  printz( "My move is: %s", mvstr[0]);
  if (beep)
    putchar (7);
  ClrEoln ();

  gotoXY(48,24);
  if (root->flags & draw)
    printz( "Drawn game!");
  else if (root->score == -9999)
    printz( "opponent mates!");
  else if (root->score == 9998)
    printz( "computer mates!");
  else if (root->score < -9000)
    printz( "opponent will soon mate!");
  else if (root->score > 9000)
    printz( "computer will soon mate!");
  ClrEoln ();

  if (post)
    {
      gotoXY(48,21);
      printz( "Nodes=  %8ld", NodeCnt);
		ClrEoln ();
      gotoXY(48,22);
      printz( "Nodes/Sec= %5ld", evrate);
      ClrEoln ();
      gotoXY(48,23);
      for (i = 1999; i >= 0 && Tree[i].f == 0 && Tree[i].t == 0; i--) ;
      printz( "Max Tree= %5d", i);
      ClrEoln ();
    }
}

void Px
ElapsedTime (iop)
     short int iop;
/*
   Determine the time that has passed since the search was started. If the
   elapsed time exceeds the target (ResponseTime+ExtraTime) then set timeout
   to true which will terminate the search.
*/


{
  et = time ((long *) 0) - time0;
  if (et < 0)
    et = 0;
  ETnodes += 50;
  if (et > et0 || iop == 1)
    {
      if (et > ResponseTime + ExtraTime && Sdepth > 1)
          timeout = true;
      et0 = et;
      if (iop == 1)
          {
           time0 = time ((long *) 0);
           et0 = 0;
          }
      if (et > 0)
          /* evrate used to be Nodes / cputime I dont` know why */
         evrate = NodeCnt / (et + ft);
      else
         evrate = 0;
      ETnodes = NodeCnt + 50;
      UpdateClocks ();
    }
}

void Px
UpdateClocks ()
{
  short m, s;
  m = et / 60;
  s = (et - 60 * m);
  if (TCflag)
    {
      m = (TimeControl.clock[player] - et) / 60;
      s = TimeControl.clock[player] - et - 60 * m;
    }
  if (m < 0)
    m = 0;
  if (s < 0)
    s = 0;
  if (player == white)
    if (reverse)
#ifdef INT10
    {
     r1.h.dh=2;r1.h.dl=20;int86(0x10,&r1,&r2);
	 }
#else
      gotoXY ( 20, 2) ;
#endif
    else
      gotoXY(20,23);
  else if (reverse)
    gotoXY(20,23);
  else
    gotoXY(20,2);
  printz( "%d:%2d   ", m, s);
  if (post)
    {
      gotoXY(48,21);
      printz( "Nodes=  %8ld", NodeCnt);
      gotoXY(48,22);
      printz( "Nodes/Sec= %5ld", evrate);
    }
  refresh ();
}

void Px
SetTimeControl ()
{
  if (TCflag)
    {
      TimeControl.moves[white] = TimeControl.moves[black] = TCmoves;
      TimeControl.clock[white] = TimeControl.clock[black] = 60 * (long) TCminutes;
    }
  else
    {
      TimeControl.moves[white] = TimeControl.moves[black] = 0;
      TimeControl.clock[white] = TimeControl.clock[black] = 0;
      Level = 60 * (long) TCminutes;
    }
  et = 0;
  ElapsedTime (1);
}

void Px
gotoXY (x, y)
     short int x;
     short int y;
{
#ifdef MSDOS
#ifdef INT10
   r1.h.dh=(char)y;r1.h.dl=(char)x;int86(0x10,&r1,&r2);
     /* inline micro expansion */
#else
  putchar(ESC);
  putchar('[');
  param(y);
  putchar(';');
  param(x);
  putchar('H');
#endif  /*not int10 but ansi  */
#else
  move (y - 1, x - 1);
#endif /* MSDOS */
}

void Px
ClrScreen ()
{
#ifdef MSDOS
#ifdef INT10
  gotoXY(0,0);
  clrln.x.cx = 2000 ;   /* 80x25 spaces to fill */
  int86(0x10,&clrln,&r2);
#else        /* use ansi.sys */
  putchar(ESC);
  putchar('[');
  putchar('2');
  putchar('J');
#endif       /* int10 or ansi */
#else   /* msdos not defined */
  clear ();
  refresh ();
#endif /* MSDOS */
}

void Px
ClrEoln ()
{
#ifdef MSDOS
#ifdef INT10
  int86(0x10,&getpos,&r2);
  clrln.x.cx = 79 - (unsigned short) r2.h.dl ;   /* number of spaces to fill */
  int86(0x10,&clrln,&r2);
#else   /* is ansi */
  putchar(ESC);
  putchar('[');
  putchar('K');
#endif   /*int10 or ansii */
#else        /* not msdos */
  clrtoeol ();
  refresh ();
#endif /* MSDOS */
}

#ifdef MSDOS
#ifndef INT10    /* only used with ansi.sys */
inline Px param(n)
short n;
{
  if (n >= 10) {
    REGISTER short d, q;
    q = n/10; d = n%10;
    putchar(q + '0');
    putchar(d + '0');
  }else
    putchar(n + '0');
}
#endif
#endif /* MSDOS */

void Px
DrawPiece (sq)
     short int sq;
{
  REGISTER short r, c;
  char x;
  if (reverse)
    r = 7 - row (sq);
  else
    r = row (sq);
  if (reverse)
    c = 7 - column (sq);
  else
    c = column (sq);
  if (color[sq] == black)
    x = '*';
  else
    x = ' ';
  gotoXY(5+5*c,5+2*(7-r));
  printz( "%c%c ", x, pxx[board[sq]]);
}

void Px
UpdateDisplay (f, t, flag, iscastle)
     short int f;
     short int t;
     short int flag;
     short int iscastle;
{
  short i, l, z;
  if (flag)
	 {
		gotoXY(48,2);
		printz( "%s %s",gname, ver);
		gotoXY(48,3);
		printz("%s %d", bk_title, numbooks);
		if (hashflag)
		 {  printz(" Hash");
			 if ( hashfile != NULL )
			 printz("+");
		 }
		 else printz(" -Hash");
		i = 3;
		gotoXY(3,++i);
		printz( "|----|----|----|----|----|----|----|----|");
		while (i < 19)
	{
		gotoXY(1,++i);
	  if (reverse)
		 z = (i / 2) - 1;
	  else
		 z = 10 - (i / 2);
		printz( "%d |    |    |    |    |    |    |    |    |", z);
		gotoXY(3,++i);
	  if (i < 19)
        printz( "+----+----+----+----+----+----+----+----+");
	}
      printz( "|----|----|----|----|----|----|----|----|");
      gotoXY(3,21);
      if (reverse)
	 printz( "  h    g    f    e    d    c    b    a");
      else
    printz( "  a    b    c    d    e    f    g    h");
      if (reverse)
    gotoXY(5,23);
      else
    gotoXY(5,2);
      if (computer == black)
    printz( "Computer");
      else
    printz( "Human   ");
      if (reverse)
    gotoXY(5,2);
      else
    gotoXY(5,23);
      if (computer == white)
    printz( "Computer");
      else
    printz( "Human   ");
      for (l = 0; l < 64; l++)
	DrawPiece (l);
    }
  else
    {
      DrawPiece (f);
      DrawPiece (t);
      if (iscastle)
	if (t > f)
	  {
	    DrawPiece (f + 3);
	    DrawPiece (t - 1);
	  }
	else
	  {
	    DrawPiece (f - 4);
	    DrawPiece (t + 1);
	  }
    }
  refresh ();
}

void Px
GetOpenings ()
/*
   Read in the Opening Book file and parse the algebraic notation for a move
   into an unsigned integer format indicating the from and to square. Create
   a linked list of opening lines of play, with entry->next pointing to the
   next line and entry->move pointing to a chunk of memory containing the
   moves. More Opening lines of up to 256 half moves may be added to
   gnuchess.book.
   **** warning ****   dag
     this routine is called by newgame, and it mallocs memory that it
     never free()s... must change this!    dag
*/

{
  FILE *fd;
  int c, i, j, side;
  struct BookEntry Fr *entry;
  unsigned short mv, tmp[0x100];    /* need 256 possible moves */
  unsigned short Fr *mp;

  if (((fd = fopen (bk_title, "r")) != NULL) ||   /* was BOOK */
      ((strcpy(bk_title,"gnuchess.boo"),
        fd = fopen (bk_title, "r")) != NULL))
    {
      ClrScreen();   /* clear if SelectLevel displayed the board */
      Book = NULL;
      i = 0;
      numbooks = 0;
      side = white;
      while ((c = parse (fd, &mv, side)) >= 0)   /* -1 is EOF, 1 is good move,
                                                   0 newgame,2 comment continue,
                                                   3 will be title */
	if (c == 1)
	  {
	    tmp[++i] = mv;
	    side = otherside[side];
	  }
	else if (c == 0 && i > 0)
	  {
#ifdef MSDOS
        entry = (struct BookEntry Fr *) malloc (sizeof (struct BookEntry));
        mp = (unsigned short Fr *) malloc ((i + 1) * sizeof (unsigned short));
#else
        entry = (struct BookEntry *) malloc (sizeof (struct BookEntry));
        mp = (unsigned short *) malloc ((i + 1) * sizeof (unsigned short));
#endif
        if ( mp == NULL || entry == NULL )
         { printz( "\n out of memory for book\n");
           exit(1);
         }
        if (vicious)
           numbooks++;
        else   /* ok to show book */
          printz("%d %s",numbooks++,cur_book); /* list book title and bump
                   counter,  no newline to allow the book to do it  dag */
        entry->mv = mp;
	    entry->next = Book;
	    Book = entry;
	    for (j = 1; j <= i; j++)
	      *(mp++) = tmp[j];
	    *mp = 0;
	    i = 0;
	    side = white;
	  }
      else continue ;   /* case for c is not 1  dag */

      fclose (fd);
      ClrScreen();   /* added this to clear books read in dag */
    }
    else strcpy(bk_title,"no book");   /* blank out title for display */
}

int  Px
parse (fd, mv, side)
     FILE *fd;
     short unsigned int *mv;
     short int side;
{
  /* returns 1  valid move, -1  EOF,
     (unused now) 2 found comment,  3 found title */
  int c, i, r1, r2, c1, c2;
  char s[100];

  while ((c = getc (fd)) == ' ') ;    /* eat opening white spaces */
  i = 0;                              /* init counter */
  s[0] = c;                           /* transfer first char */
  while (c != ' ' && c != '\n' && c != EOF )   /* limits */
     s[++i] = c = getc (fd);                /* read in temp */
  s[++i] = '\0';
  if (c == EOF)
    return (-1);
  /*if (s[0] == '!' || s[0] == ';' || i < 3)   old way  dag */
  if (s[0] == '!' )      /* signal new game, i < 3 indicates a bad
                          move and starts a new book too.. a bug removed dag*/
    {
      while (c != '\n' && c != EOF && c != 'T')
            c = getc (fd);
      if (c == 'T')             /* Title trigger */
       { i = 0;                            /* reuse i */
         while (c != '\n' && c != EOF)
            cur_book[i++] = c = getc (fd);      /* read in title */
         cur_book[i] = '\0';              /* terminate string, overwrite \n */
                                           /* later copy string and
                                              return a 3 for use */
         return (0);                         /* return title value */
       }
      else return (0);
    }
    if (s[0] == ';' || i < 3 )                 /* comment trigger */
    {                                     /* with i < 3 for now
                                    this eats strange lines and bad
                                     moves but it is not elegant    dag*/
      while (c != '\n' && c != EOF)
            c = getc (fd);                 /* eat comment for now */
      return (2);                          /* return comment value */
    }

  if (s[4] == 'o')
    if (side == black)
      *mv = 0x3C3A;
    else
      *mv = 0x0402;
  else if (s[0] == 'o')
    if (side == black)
      *mv = 0x3C3E;
    else
      *mv = 0x0406;
  else
    {
      c1 = s[0] - 'a';
      r1 = s[1] - '1';
      c2 = s[2] - 'a';
      r2 = s[3] - '1';
      *mv = (locn (r1, c1) << 8) + locn (r2, c2);
    }
  return (1);
}

void Px
GetGame ()
{
  FILE *fd;
  char fname[40];
  int c;
  short sq;
  unsigned short m;

  ShowMessage ("Enter file name: ");
  scanz ("%s", fname);
  if (fname[0] == '\0')
    strcpy (fname, "chess.000");
  if ((fd = fopen (fname, "r")) != NULL)
    {
      fscanf (fd, "%hd%hd%hd", &computer, &opponent, &Game50);
      fscanf (fd, "%hd%hd", &castld[white], &castld[black]);
      fscanf (fd, "%hd%hd", &TCflag, &OperatorTime);
      fscanf (fd, "%ld%ld%hd%hd",
	      &TimeControl.clock[white], &TimeControl.clock[black],
	      &TimeControl.moves[white], &TimeControl.moves[black]);
      for (sq = 0; sq < 64; sq++)
	{
	  fscanf (fd, "%hd%hd", &m, &Mvboard[sq]);
	  board[sq] = (m >> 8);
	  color[sq] = (m & 0xFF);
	  if (color[sq] == 0)
	    color[sq] = neutral;
	  else
	    --color[sq];
	}
      GameCnt = -1;
      c = '?';
      while (c != EOF)
	{
	  ++GameCnt;
	  c = fscanf (fd, "%hd%hd%hd%ld%hd%hd%hd", &GameList[GameCnt].gmove,
		      &GameList[GameCnt].score, &GameList[GameCnt].depth,
		      &GameList[GameCnt].nodes, &GameList[GameCnt].time,
		      &GameList[GameCnt].piece, &GameList[GameCnt].color);
	  if (GameList[GameCnt].color == 0)
	    GameList[GameCnt].color = neutral;
	  else
	    --GameList[GameCnt].color;
	}
      GameCnt--;
      if (TimeControl.clock[white] > 0)
	TCflag = true;
      computer--;
      opponent--;
    }
  fclose (fd);
  InitializeStats ();
  UpdateDisplay (0, 0, 1, 0);
  Sdepth = 0;
}



void Px
SaveGame ()
{
  FILE *fd;
  char fname[40], jnkstr[40];
  short sq, i, c;

  ShowMessage ("Enter file name: ");
  scanz ("%s", fname);
  gotoXY(48,23);                /* reset for message */

  if (fname[0] == '\0')
    strcpy (fname, "chess.000");
  if ((fd = fopen (fname, "r")) != NULL)
    { fclose (fd);
      ShowMessage ("OverWrite file?");
      scanz ("%s",jnkstr);
      surelow (jnkstr);
      if (jnkstr[0] != 'y')
         return;
     }
  if ((fd = fopen (fname, "w")) != NULL)
    {
      fprintf (fd, "%d %d %d\n", computer + 1, opponent + 1, Game50);
      fprintf (fd, "%d %d\n", castld[white], castld[black]);
      fprintf (fd, "%d %d\n", TCflag, OperatorTime);
      fprintf (fd, "%ld %ld %d %d\n",
	       TimeControl.clock[white], TimeControl.clock[black],
	       TimeControl.moves[white], TimeControl.moves[black]);
      for (sq = 0; sq < 64; sq++)
       {
        if (color[sq] == neutral)
             c = 0;
        else
            c = color[sq] + 1;
        fprintf (fd, "%d %d\n", 256 * board[sq] + c, Mvboard[sq]);
       }
      for (i = 0; i <= GameCnt; i++)
       {
        if (GameList[i].color == neutral)
             c = 0;
        else
             c = GameList[i].color + 1;
        fprintf (fd, "%d %d %d %ld %d %d %d\n",
		   GameList[i].gmove, GameList[i].score, GameList[i].depth,
		   GameList[i].nodes, GameList[i].time,
		   GameList[i].piece, c);
       }
      fclose (fd);
      printz( "Game saved on file: %s", fname);
    }
  else
    printz( "Could not open file: %s", fname);
}

void Px
ListGame ()
{
  FILE *fd;
  short i, f, t;
  fd = fopen ("chess.lst", "w");
  fprintf (fd, "\n");
  fprintf (fd, "       score  depth   nodes  time         ");
  fprintf (fd, "       score  depth   nodes  time\n");
  for (i = 0; i <= GameCnt; i++)
    {
      f = GameList[i].gmove >> 8;
      t = (GameList[i].gmove & 0xFF);
      algbr (f, t, false);
      if ((i % 2) == 0)
	fprintf (fd, "\n");
      else
	fprintf (fd, "         ");
      fprintf (fd, "%5s  %5d     %2d %7ld %5d", mvstr[0],
	       GameList[i].score, GameList[i].depth,
	       GameList[i].nodes, GameList[i].time);
    }
  fprintf (fd, "\n\n");
  fclose (fd);
  ShowMessage ("chess.lst written");
}

void Px
Undo ()
/*
   Undo the most recent half-move.
*/

{
  short f, t;
  f = GameList[GameCnt].gmove >> 8;
  t = GameList[GameCnt].gmove & 0xFF;
  if (board[t] == king && distance (t, f) > 1)
    (void) castle (GameList[GameCnt].color, f, t, 2);
  else
    {
      /* Check for promotion: */
      if ((color[t] == white && row (f) == 6 && row (t) == 7)
	  || (color[t] == black && row (f) == 1 && row (t) == 0))
	{
	  int g, from = f;
	  for (g = GameCnt - 1; g >= 0; g--)
	    if ((GameList[g].gmove & 0xFF) == from) /* added parantheses  yde */
	      from = GameList[g].gmove >> 8;
	  if ((color[t] == white && row (from) == 1)
	      || (color[t] == black && row (from) == 6))
	    board[t] = pawn;
	}

      board[f] = board[t];
      color[f] = color[t];
      board[t] = GameList[GameCnt].piece;
      color[t] = GameList[GameCnt].color;
      if (color[t] != neutral)
	Mvboard[t]--;
      Mvboard[f]--;
    }
  if (TCflag)
    ++TimeControl.moves[color[f]];
  GameCnt--;
  computer = otherside[computer];
  opponent = otherside[opponent];
  mate = false;
  Sdepth = 0;
  UpdateDisplay (0, 0, 1, 0);
  InitializeStats ();
}


void Px
ShowSidetomove ()
{
  gotoXY(48,14);
  if (player == white)
    printz( "%2d:   WHITE", 1 + (GameCnt + 1) / 2);
  else
    printz( "%2d:   BLACK", 1 + (GameCnt + 1) / 2);
  ClrEoln ();
}

void Px
PromptForMove (passed)
short passed;
{
  gotoXY(48,19);
#ifdef CHECKANO
  if (InChk)   printz( "Check! ");
#endif
  if (passed) printz("Thinking, ^C to interrupt");
    else      printz("Your move is? ");
  ClrEoln ();
}

void Px
ShowCurrentMove (pnt, f, t)
     short int pnt;
     short int f;
     short int t;
{
  algbr (f, t, false);
  gotoXY(48,7);
  printz( "(%2d) %4s", pnt, mvstr[0]);
}

void Px
ChangeAlphaWindow ()
{
  ShowMessage ("A window: ");
  scanz ("%hd", &Awindow);
}

void Px
ChangeBetaWindow ()
{
  ShowMessage ("B window: ");
  scanz ("%hd", &Bwindow);
}

void Px
GiveHint ()
{
  char s[40];
  algbr ((short) (hint >> 8), (short) (hint & 0xFF), false);
  strcpy (s, "try ");
  strcat (s, mvstr[0]);
  ShowMessage (s);
}

void Px
ChangeSearchDepth ()
{
  ShowMessage ("depth= ");
  scanz ("%hd", &MaxSearchDepth);
}

void Px
SetContempt ()
{
  ShowMessage ("contempt= ");
  scanz ("%hd", &contempt);
}

void Px
SetHprobe()
{
   ShowMessage ("rehash= ");
   scanz ("%hd", &rehash);

}

void Px
ChangeXwindow ()
{
  ShowMessage ("xwndw= ");
  scanz ("%hd", &xwndw);
}

void Px
FileAccess()   /* select file access level   dag */
{
  ShowMessage ("HashFile access after= ");
  scanz ("%hd", &ToFileAt);
}

void Px
SelectLevel ()
{
  char s[30];

  ClrScreen ();
  gotoXY(20,2);
  printz( "Welcome to %s %s",gname, ver );
  gotoXY(20,4);
  printz( " 1.   60 moves in   5 minutes");
  gotoXY(20,5);
  printz( " 2.   60 moves in  15 minutes");
  gotoXY(20,6);
  printz( " 3.   60 moves in  30 minutes");
  gotoXY(20,7);
  printz( " 4.   40 moves in  30 minutes");
  gotoXY(20,8);
  printz( " 5.   40 moves in  60 minutes");
  gotoXY(20,9);
  printz( " 6.   40 moves in 120 minutes");
  gotoXY(20,10);
  printz( " 7.   40 moves in 240 minutes");
  gotoXY(20,11);
  printz( " 8.    1 move  in  15 minutes");
  gotoXY(20,12);
  printz( " 9.    1 move  in  60 minutes");
  gotoXY(20,13);
  printz( "10.    1 move  in 600 minutes");
  gotoXY(20,14);
  printz( "11.    select  moves  in  minutes");

  OperatorTime = 1;
  TCmoves = 60;
  TCminutes = 5;
  MaxSearchDepth= 29;

  gotoXY(20,18);
  printz( "Enter Level: ");
  refresh ();
  scanz ("%ld", &Level);
  switch ((int)Level)
    {
    case 1:
      TCmoves = 60;
      TCminutes = 5;
      break;
    case 2:
      TCmoves = 60;
      TCminutes = 15;
      break;
    case 3:
      TCmoves = 60;
      TCminutes = 30;
      break;
    case 4:
      TCmoves = 40;
      TCminutes = 30;
      break;
    case 5:
      TCmoves = 40;
      TCminutes = 60;
      break;
    case 6:
      TCmoves = 40;
      TCminutes = 120;
      break;
    case 7:
      TCmoves = 40;
      TCminutes = 240;
      break;
    case 8:
      TCmoves = 1;
      TCminutes = 15;
      break;
    case 9:
      TCmoves = 1;
      TCminutes = 60;
      break;
    case 10:
      TCmoves = 1;
      TCminutes = 600;
      break;
    case 11:                  /* may need to add level adjusts dag */
      ClrScreen();
      gotoXY(20,10);
      printz("Enter number of moves: ");
      refresh ();
      scanz ("%d", &TCmoves);
      gotoXY(20,12);
      printz("Enter number of minutes: ");
      refresh ();
      scanz ("%d", &TCminutes);
      gotoXY(20,14);
      printz("Enter maximum search Depth: 1 to [29] ");
      refresh ();
      scanz ("%hd", &MaxSearchDepth);
        /* try resetting level lower to trigger quiet search */
        if (TCmoves == 1)

            OperatorTime = 0;  /* done in command line version, factors
                                  in timecontrol, not sure of effect  dag*/
#if 0      /* taken out for a while.  This will be a place to place the
             clutch for agressive/passive search when I figure it out  dag*/
        else    /* more than 1 move per time period */
        {  gotoXY(20,16);
           printz("Set quiet search, default is %ld\n",Level);
           gotoXY(20,17);
           printz("new value: ");
           refresh ();
           scanz ("%ld", &Level);
        }
#endif
      gotoXY(20,16);
      printz("Am I Mr. Niceguy? ");
      refresh ();
      scanz ("%s", s );
      surelow(s);
      if (strcmp(s,"yes") == 0)
           vicious = 0;
      else vicious = 1;

      break;

    }

  TCflag = (TCmoves > 1);
  SetTimeControl ();
  ClrScreen ();
  UpdateDisplay (0, 0, 1, 0);
}

void Px
ShowPostnValues ()
{
  short i, r, c;
  ExaminePosition ();
  for (i = 0; i < 64; i++)
    {
      if (reverse)
	r = 7 - row (i);
      else
	r = row (i);
      if (reverse)
	c = 7 - column (i);
      else
	c = column (i);
      gotoXY(4+5*c,5+2*(7-r));
      c1 = color[i];
      c2 = otherside[c1];
      PC1 = PawnCnt[c1];
      PC2 = PawnCnt[c2];
      atk1 = atak[c1];
      atk2 = atak[c2];
      if (color[i] == neutral)
    printz( "   ");
      else
    printz( "%3d ", SqValue (i, opponent));
    }
  ScorePosition (opponent, &i);
  gotoXY(48,24);
  printz( "Score= %d", i);
  ClrEoln ();
}

void Px
DoDebug ()
{
  short k, p, i, r, c, tp, tc;
  char s[40];

  ExaminePosition ();
  ShowMessage ("Enter piece: ");
  scanz ("%s", s);
  surelow(s);

  if (s[0] == 'w')
    k = white;
  else
    k = black;
  if (s[1] == 'p')
    p = pawn;
  else if (s[1] == 'n' || s[1] == 'N')
    p = knight;
  else if (s[1] == 'b' || s[1] == 'B')
    p = bishop;
  else if (s[1] == 'r' || s[1] == 'R')
    p = rook;
  else if (s[1] == 'q' || s[1] == 'Q')
    p = queen;
  else if (s[1] == 'k' || s[1] == 'K')
    p = king;
  else
    p = no_piece;
  for (i = 0; i < 64; i++)
    {
      if (reverse)
	r = 7 - row (i);
      else
	r = row (i);
      if (reverse)
	c = 7 - column (i);
      else
	c = column (i);
      gotoXY(4+5*c,5+2*(7-r));
      tp = board[i];
      tc = color[i];
      board[i] = p;
      color[i] = k;
      c1 = k;
      c2 = otherside[c1];
      PC1 = PawnCnt[c1];
      PC2 = PawnCnt[c2];
      atk1 = atak[c1];
      atk2 = atak[c2];
      printz( "%3d ", SqValue (i, opponent));
      board[i] = tp;
      color[i] = tc;
    }
  ScorePosition (opponent, &i);
  gotoXY(48,24);
  printz( "Score= %d", i);
  ClrEoln ();
}


void Px
InputCommand ()
/*
   Process the users command. If easy mode is OFF (the computer is thinking
   on opponents time) and the program is out of book, then make the 'hint'
   move on the board and call SelectMove() to find a response. The user
   terminates the search by entering ^C (quit siqnal) before entering a
   command. If the opponent does not make the hint move, then set Sdepth to
   zero.
*/

{
  short ok, i, tmp;
  long cnt, rate, t1, t2;
  unsigned short mv;
  ok = quit = false;
  /* removed s[80] to global and renamed str[80] */
  player = opponent;
  ShowSidetomove ();
  ft = 0;

  if (hint > 0 && !easy && Book == NULL)
    {
      fflush (stdout);
      time0 = time ((long *) 0);
      algbr ((short) hint >> 8, (short) hint & 0xFF, false);
      strcpy (str, mvstr[0]);
      tmp = epsquare;
      if (VerifyMove (str, 1, &mv))
	{
      PromptForMove (1);      /* just shows message while -a active */
	  SelectMove (computer, 2);
	  (void) VerifyMove (mvstr[0], 2, &mv);
	  if (Sdepth > 0)
	    Sdepth--;
	}
      ft = time ((long *) 0) - time0;
      epsquare = tmp;
    }
  signal (SIGINT, Die);
#ifndef MSDOS
  signal (SIGQUIT, Die);
#endif /* MSDOS */

  while (!(ok || quit))
    {
      PromptForMove (0);         /* actual input routine */
      scanz ("%s", str);
      surelow(str);
      player = opponent;
      ok = VerifyMove (str, 0, &mv);
      if (ok && mv != hint)
       {
        Sdepth = 0;
        ft = 0;
       }
      if (*str == '\0')
          UpdateDisplay (0, 0, 1, 0);
      if (strcmp (str, "edit") == 0)
          EditBoard (0);
      if (strcmp (str, "position") == 0)
           EditBoard (1);
      if (strcmp (str, "bd") == 0)
       {
        ClrScreen ();
        UpdateDisplay (0, 0, 1, 0);
       }
      if (strcmp (str, "quit") == 0)
          quit = true;
      if (strcmp (str, "lurk") == 0)     /* added for testing  dag*/
       {    if (!post)
                post = !post;   /* turn on if off */
            if (beep)           /* keep the bell off */
                 beep = !beep;
            strcpy (str, "both");  /* fake command to init playing both sides*/
        }
      if (strcmp (str, "post") == 0)
          post = !post;
      /* edit and position were here, moved up incase string is used */
      if (strcmp (str, "go") == 0)
          ok = true;
      if (strcmp (str, "help") == 0)
          help ();
      if (strcmp (str, "force") == 0)
          force = !force;
      if (strcmp (str, "book") == 0)
          Book = NULL;
      if (strcmp (str, "undo") == 0 && GameCnt >= 0)
          Undo ();
      if (strcmp (str, "new") == 0)
          NewGame ();
      if (strcmp (str, "list") == 0)
          ListGame ();
      if (strcmp (str, "level") == 0)
          SelectLevel ();
      if (strcmp (str, "tofile") == 0)
          FileAccess();
      if (strcmp (str, "hash") == 0)
         {
           if (hashfile)
            {  fclose(hashfile);
               hashfile = NULL;            /* must zero out references! dag */
               ToFileAt = maxdepth;        /* set of max if table only  */
            }
           else
              if ((hashfile = fopen ("hashtest", "r")) == NULL)
                    ShowMessage("No file to use!");
               else{
                    fclose(hashfile);
#ifdef MSDOS                         /* error check not needed  dag */
                    hashfile = fopen ("hashtest", "wb+");
#else
                    hashfile = fopen ("hashtest", "w+");
#endif

                    ToFileAt = 4;
                   }

          }
      if (strcmp (str, "beep") == 0)
          beep = !beep;
      if (strcmp (str, "awindow") == 0)
          ChangeAlphaWindow ();
      if (strcmp (str, "bwindow") == 0)
          ChangeBetaWindow ();
      if (strcmp (str, "hint") == 0)
          GiveHint ();
      if (strcmp (str, "both") == 0)
       {
        bothsides = !bothsides;
        Sdepth = 0;
        force = false;  /* added to keep both sides playing after force used
                         to setup game to test */
        SelectMove (opponent, 1);
        ok = true;
       }
      if (strcmp (str, "reverse") == 0)
       {
        reverse = !reverse;
        ClrScreen ();
        UpdateDisplay (0, 0, 1, 0);
       }
      if (strcmp (str, "switch") == 0)
       {
        computer = otherside[computer];
        opponent = otherside[opponent];
        force = false;
        Sdepth = 0;
        ok = true;
       }
      if (strcmp (str, "white") == 0)
       {
        computer = white;
        opponent = black;
        ok = true;
        force = false;
        Sdepth = 0;
       }
      if (strcmp (str, "black") == 0)
       {
        computer = black;
        opponent = white;
        ok = true;
        force = false;
        Sdepth = 0;
       }
      if (strcmp (str, "remove") == 0 && GameCnt >= 1)
       {
        Undo ();
        Undo ();
       }
      if (strcmp (str, "get") == 0)
          GetGame ();
      if (strcmp (str, "save") == 0)
          SaveGame ();
      if (strcmp (str, "depth") == 0)
          ChangeSearchDepth ();
      if (strcmp (str, "random") == 0)
          dither = 6;
      if (strcmp (str, "easy") == 0)
          seteasy(1);                      /* call with reset for ahead  dag*/
      if (strcmp (str, "contempt") == 0)
          SetContempt ();
      if (strcmp (str, "rehash") == 0)
          SetHprobe ();
      if (strcmp (str, "xwndw") == 0)
          ChangeXwindow ();
      if (strcmp (str, "test") == 0)
       {
        t1 = time (0);
        cnt = 0;
        for (i = 0; i < 10000; i++)
	    {
	      MoveList (opponent, 2);
	      cnt += TrPnt[3] - TrPnt[2];
	    }
        t2 = time (0);
        rate = cnt / (t2 - t1);
        gotoXY(48,24);
        printz( "cnt= %ld  rate= %ld", cnt, rate);
        ClrEoln ();
       }
      if (strcmp (str, "p") == 0)
          ShowPostnValues ();
      if (strcmp (str, "debug") == 0)
          DoDebug ();
    }

  ClearMessage ();
  ElapsedTime (1);
  if (force)
    {
      computer = opponent;
      opponent = otherside[computer];
    }
  signal (SIGINT, (TerminateSearch));
#ifndef MSDOS
  signal (SIGQUIT, TerminateSearch);
#endif /* MSDOS */
}


